@@ -1,7 +1,7 @@
 #ifndef STAN_MODEL_HESSIAN_HPP
 #define STAN_MODEL_HESSIAN_HPP
 
-#include <stan/math/mix/mat.hpp>
+#include <stan/math/mix.hpp>
 #include <stan/model/model_functional.hpp>
 #include <iostream>
 
