@@ -13,12 +13,13 @@
 #include <boost/circular_buffer.hpp>
 #include <boost/lexical_cast.hpp>
 #include <algorithm>
+#include <chrono>
 #include <limits>
 #include <numeric>
 #include <ostream>
-#include <vector>
 #include <queue>
 #include <string>
+#include <vector>
 
 namespace stan {
 
@@ -86,7 +87,7 @@ class advi {
    * @return the evidence lower bound.
    * @throw std::domain_error If, after n_monte_carlo_elbo_ number of draws
    * from the variational distribution all give non-finite log joint
-   * evaluations. This means that the model is severly ill conditioned or
+   * evaluations. This means that the model is severely ill conditioned or
    * that the variational distribution has somehow collapsed.
    */
   double calc_ELBO(const Q& variational, callbacks::logger& logger) const {
@@ -115,8 +116,8 @@ class advi {
           const char* msg2
               = "). Your model may be either severely "
                 "ill-conditioned or misspecified.";
-          stan::math::domain_error(function, name, n_monte_carlo_elbo_, msg1,
-                                   msg2);
+          stan::math::throw_domain_error(function, name, n_monte_carlo_elbo_,
+                                         msg1, msg2);
         }
       }
     }
@@ -187,7 +188,7 @@ class advi {
       const char* msg1
           = "Your model may be either "
             "severely ill-conditioned or misspecified.";
-      stan::math::domain_error(function, name, "", msg1);
+      stan::math::throw_domain_error(function, name, "", msg1);
     }
 
     // Variational family to store gradients
@@ -279,7 +280,7 @@ class advi {
             const char* msg1
                 = "failed. Your model may be either "
                   "severely ill-conditioned or misspecified.";
-            stan::math::domain_error(function, name, "", msg1);
+            stan::math::throw_domain_error(function, name, "", msg1);
           }
         }
         // Reset
@@ -294,7 +295,7 @@ class advi {
   /**
    * Runs stochastic gradient ascent with an adaptive stepsize sequence.
    *
-   * @param[in,out] variational initia variational distribution
+   * @param[in,out] variational initial variational distribution
    * @param[in] eta stepsize scaling parameter
    * @param[in] tol_rel_obj relative tolerance parameter for convergence
    * @param[in] max_iterations max number of iterations to run algorithm
@@ -347,9 +348,7 @@ class advi {
         "   notes ");
 
     // Timing variables
-    clock_t start = clock();
-    clock_t end;
-    double delta_t;
+    auto start = std::chrono::steady_clock::now();
 
     // Main loop
     bool do_more_iterations = true;
@@ -388,10 +387,11 @@ class advi {
            << std::setw(16) << std::fixed << std::setprecision(3)
            << delta_elbo_ave << "  " << std::setw(15) << std::fixed
            << std::setprecision(3) << delta_elbo_med;
-
-        end = clock();
-        delta_t = static_cast<double>(end - start) / CLOCKS_PER_SEC;
-
+        auto end = std::chrono::steady_clock::now();
+        double delta_t
+            = std::chrono::duration_cast<std::chrono::milliseconds>(end - start)
+                  .count()
+              / 1000.0;
         std::vector<double> print_vector;
         print_vector.clear();
         print_vector.push_back(iter_counter);
