@@ -2,7 +2,7 @@
 #define STAN_MATH_FWD_CORE_FVAR_HPP
 
 #include <stan/math/prim/meta.hpp>
-#include <stan/math/prim/scal/fun/is_nan.hpp>
+#include <stan/math/prim/fun/is_nan.hpp>
 #include <ostream>
 #include <type_traits>
 
@@ -11,19 +11,18 @@ namespace math {
 
 /**
  * This template class represents scalars used in forward-mode
- * automatic differentiation, which consist of values and
- * directional derivatives of the specified template type.  When
- * performing operations on instances of this class, all operands
- * should be either primitive integer or double values or dual
- * numbers representing derivatives in the same direction.  The
- * typical use case is to have a unit length directional
- * derivative in the direction of a single independent variable.
+ * automatic differentiation, which consist of values and directional
+ * derivatives of the specified template type.  When performing
+ * operations on instances of this class, all operands should be
+ * either primitive integer or double values or dual numbers
+ * representing derivatives in the same direction.  The typical use
+ * case is to have a unit length directional derivative in the
+ * direction of a single independent variable.
  *
- * By using reverse-mode automatic derivative variables,
- * second-order derivatives may
- * be calculated.  By using fvar&lt;<var&gt; instances,
- * third-order derivatives may be calculated.  These are called
- * mixed-mode automatic differentiation variable in Stan.
+ * By using reverse-mode automatic derivative variables, second-order
+ * derivatives may be calculated.  By using fvar&lt;var&gt;
+ * instances, third-order derivatives may be calculated.  These are
+ * called mixed-mode automatic differentiation variable in Stan.
  *
  * Specialized functionals that perform differentiation on
  * functors may be found in the matrix subdirectories of the
@@ -31,55 +30,47 @@ namespace math {
  *
  * The <a
  * href="https://en.wikipedia.org/wiki/Automatic_differentiation">Wikipedia
- * page on automatic differentiation</a> describes how
- * forward-mode automatic differentiation works mathematically in
- * terms of dual numbers.
+ * page on automatic differentiation</a> describes how forward-mode
+ * automatic differentiation works mathematically in terms of dual
+ * numbers.
  *
  * @tparam T type of value and tangent
  */
 template <typename T>
 struct fvar {
+  /**
+   * The type of values and tangents.
+   */
+  using Scalar = std::decay_t<T>;
+
   /**
    * The value of this variable.
    */
-  T val_;
-
-  /**
-   * The tangent (derivative) of this variable.
-   */
-  T d_;
-
-  /**
-   * The Type inside of the fvar.
-   */
-  using Scalar = T;
+  Scalar val_;
 
   /**
    * Return the value of this variable.
    *
    * @return value of this variable
    */
-  T val() const { return val_; }
+  Scalar val() const { return val_; }
+
+  /**
+   * The tangent (derivative) of this variable.
+   */
+  Scalar d_;
 
   /**
    * Return the tangent (derivative) of this variable.
    *
    * @return tangent of this variable
    */
-  T tangent() const { return d_; }
+  Scalar d() const { return d_; }
 
   /**
    * Construct a forward variable with zero value and tangent.
    */
-  fvar() : val_(0), d_(0) {}
-
-  /**
-   * Construct a forward variable with value and tangent set to
-   * the value and tangent of the specified variable.
-   *
-   * @param[in] x variable to be copied
-   */
-  fvar(const fvar<T>& x) : val_(x.val_), d_(x.d_) {}
+  fvar() : val_(0.0), d_(0.0) {}  // NOLINT
 
   /**
    * Construct a forward variable with the specified value and
@@ -88,17 +79,16 @@ struct fvar {
    * @tparam V type of value (must be assignable to T)
    * @param[in] v value
    */
-  template <typename V, typename = std::enable_if_t<ad_promotable<V, T>::value>>
+  template <typename V, std::enable_if_t<ad_promotable<V, T>::value>* = nullptr,
+            require_not_same_t<V, fvar<T>>* = nullptr>
   fvar(const V& v) : val_(v), d_(0) {}  // NOLINT(runtime/explicit)
 
   /**
    * Construct a forward variable with the specified value and
    * tangent.
    *
-   * @tparam V type of value (must be assignable to the value and
-   *   tangent type T)
-   * @tparam D type of tangent (must be assignable to the value and
-   *   tangent type T)
+   * @tparam V type of value (must be assignable to class type T)
+   * @tparam D type of tangent (must be assignable class type T)
    * @param[in] v value
    * @param[in] d tangent
    */
@@ -265,6 +255,7 @@ struct fvar {
     return os << v.val_;
   }
 };
+
 }  // namespace math
 }  // namespace stan
 #endif
