@@ -1,15 +1,15 @@
 #ifndef STAN_MATH_MIX_META_HPP
 #define STAN_MATH_MIX_META_HPP
 
-#include <stan/math/prim/scal/meta/ad_promotable.hpp>
+#include <stan/math/prim/meta/ad_promotable.hpp>
 
 #include <stan/math/fwd/core.hpp>
-#include <stan/math/fwd/scal/meta/is_fvar.hpp>
-#include <stan/math/fwd/scal/meta/partials_type.hpp>
+#include <stan/math/fwd/meta/is_fvar.hpp>
+#include <stan/math/fwd/meta/partials_type.hpp>
 
 #include <stan/math/rev/core.hpp>
-#include <stan/math/rev/scal/meta/is_var.hpp>
-#include <stan/math/rev/scal/meta/partials_type.hpp>
+#include <stan/math/rev/meta/is_var.hpp>
+#include <stan/math/rev/meta/partials_type.hpp>
 
 #include <stan/math/fwd/meta.hpp>
 #include <stan/math/rev/meta.hpp>
