@@ -1,12 +1,8 @@
-/*
- * -----------------------------------------------------------------
- * $Revision$
- * $Date$
- * -----------------------------------------------------------------
+/* -----------------------------------------------------------------
  * Programmer(s): Alan C. Hindmarsh and Radu Serban @ LLNL
  * -----------------------------------------------------------------
  * SUNDIALS Copyright Start
- * Copyright (c) 2002-2019, Lawrence Livermore National Security
+ * Copyright (c) 2002-2022, Lawrence Livermore National Security
  * and Southern Methodist University.
  * All rights reserved.
  *
@@ -17,29 +13,27 @@
  * -----------------------------------------------------------------
  * This is the implementation file for the optional input and output
  * functions for the CVODES solver.
- * -----------------------------------------------------------------
- */
+ * ----------------------------------------------------------------- */
 
 #include <stdio.h>
 #include <stdlib.h>
 
 #include "cvodes_impl.h"
-
-#include <sundials/sundials_math.h>
-#include <sundials/sundials_types.h>
+#include "sundials/sundials_types.h"
+#include "sundials/sundials_math.h"
 
 #define ZERO   RCONST(0.0)
 #define HALF   RCONST(0.5)
 #define ONE    RCONST(1.0)
 #define TWOPT5 RCONST(2.5)
 
-/* 
+/*
  * =================================================================
  * CVODES optional input functions
  * =================================================================
  */
 
-/* 
+/*
  * CVodeSetErrHandlerFn
  *
  * Specifies the error handler function
@@ -62,7 +56,7 @@ int CVodeSetErrHandlerFn(void *cvode_mem, CVErrHandlerFn ehfun, void *eh_data)
   return(CV_SUCCESS);
 }
 
-/* 
+/*
  * CVodeSetErrFile
  *
  * Specifies the FILE pointer for output (NULL means no messages)
@@ -84,7 +78,65 @@ int CVodeSetErrFile(void *cvode_mem, FILE *errfp)
   return(CV_SUCCESS);
 }
 
-/* 
+/*
+ * CVodeSetMonitorFn
+ *
+ * Specifies the user function to call for monitoring
+ * the solution and/or integrator statistics.
+ */
+
+int CVodeSetMonitorFn(void *cvode_mem, CVMonitorFn fn)
+{
+  CVodeMem cv_mem;
+
+  if (cvode_mem==NULL) {
+    cvProcessError(NULL, CV_MEM_NULL, "CVODES", "CVodeSetMonitorFn", MSGCV_NO_MEM);
+    return(CV_MEM_NULL);
+  }
+
+  cv_mem = (CVodeMem) cvode_mem;
+
+#ifdef SUNDIALS_BUILD_WITH_MONITORING
+  cv_mem->cv_monitorfun = fn;
+  return(CV_SUCCESS);
+#else
+  cvProcessError(cv_mem, CV_ILL_INPUT, "CVODES", "CVodeSetMonitorFn", "SUNDIALS was not built with monitoring enabled.");
+  return(CV_ILL_INPUT);
+#endif
+}
+
+/*
+ * CVodeSetMonitorFrequency
+ *
+ * Specifies the frequency with which to call the user function.
+ */
+
+int CVodeSetMonitorFrequency(void *cvode_mem, long int nst)
+{
+  CVodeMem cv_mem;
+
+  if (cvode_mem==NULL) {
+    cvProcessError(NULL, CV_MEM_NULL, "CVODES", "CVodeSetMonitorFrequency", MSGCV_NO_MEM);
+    return(CV_MEM_NULL);
+  }
+
+  if (nst < 0) {
+    cvProcessError(NULL, CV_ILL_INPUT, "CVODES", "CVodeSetMonitorFrequency", "step interval must be >= 0\n");
+    return(CV_ILL_INPUT);
+  }
+
+  cv_mem = (CVodeMem) cvode_mem;
+
+#ifdef SUNDIALS_BUILD_WITH_MONITORING
+  cv_mem->cv_monitor_interval = nst;
+  return(CV_SUCCESS);
+#else
+  cvProcessError(cv_mem, CV_ILL_INPUT, "CVODES", "CVodeSetMonitorFrequency", "SUNDIALS was not built with monitoring enabled.");
+  return(CV_ILL_INPUT);
+#endif
+}
+
+/*
  * CVodeSetUserData
  *
  * Specifies the user data pointer for f
@@ -106,7 +158,7 @@ int CVodeSetUserData(void *cvode_mem, void *user_data)
   return(CV_SUCCESS);
 }
 
-/* 
+/*
  * CVodeSetMaxOrd
  *
  * Specifies the maximum method order
@@ -128,7 +180,7 @@ int CVodeSetMaxOrd(void *cvode_mem, int maxord)
     cvProcessError(cv_mem, CV_ILL_INPUT, "CVODES", "CVodeSetMaxOrd", MSGCV_NEG_MAXORD);
     return(CV_ILL_INPUT);
   }
-  
+
   /* Cannot increase maximum order beyond the value that
      was used when allocating memory */
   qmax_alloc = cv_mem->cv_qmax_alloc;
@@ -145,7 +197,7 @@ int CVodeSetMaxOrd(void *cvode_mem, int maxord)
   return(CV_SUCCESS);
 }
 
-/* 
+/*
  * CVodeSetMaxNumSteps
  *
  * Specifies the maximum number of integration steps
@@ -171,7 +223,7 @@ int CVodeSetMaxNumSteps(void *cvode_mem, long int mxsteps)
   return(CV_SUCCESS);
 }
 
-/* 
+/*
  * CVodeSetMaxHnilWarns
  *
  * Specifies the maximum number of warnings for small h
@@ -193,7 +245,7 @@ int CVodeSetMaxHnilWarns(void *cvode_mem, int mxhnil)
   return(CV_SUCCESS);
 }
 
-/* 
+/*
  *CVodeSetStabLimDet
  *
  * Turns on/off the stability limit detection algorithm
@@ -220,7 +272,7 @@ int CVodeSetStabLimDet(void *cvode_mem, booleantype sldet)
   return(CV_SUCCESS);
 }
 
-/* 
+/*
  * CVodeSetInitStep
  *
  * Specifies the initial step size
@@ -242,7 +294,7 @@ int CVodeSetInitStep(void *cvode_mem, realtype hin)
   return(CV_SUCCESS);
 }
 
-/* 
+/*
  * CVodeSetMinStep
  *
  * Specifies the minimum step size
@@ -280,7 +332,7 @@ int CVodeSetMinStep(void *cvode_mem, realtype hmin)
   return(CV_SUCCESS);
 }
 
-/* 
+/*
  * CVodeSetMaxStep
  *
  * Specifies the maximum step size
@@ -320,7 +372,7 @@ int CVodeSetMaxStep(void *cvode_mem, realtype hmax)
   return(CV_SUCCESS);
 }
 
-/* 
+/*
  * CVodeSetStopTime
  *
  * Specifies the time beyond which the integration is not to proceed.
@@ -355,7 +407,7 @@ int CVodeSetStopTime(void *cvode_mem, realtype tstop)
   return(CV_SUCCESS);
 }
 
-/* 
+/*
  * CVodeSetMaxErrTestFails
  *
  * Specifies the maximum number of error test failures during one
@@ -378,10 +430,10 @@ int CVodeSetMaxErrTestFails(void *cvode_mem, int maxnef)
   return(CV_SUCCESS);
 }
 
-/* 
+/*
  * CVodeSetMaxConvFails
  *
- * Specifies the maximum number of nonlinear convergence failures 
+ * Specifies the maximum number of nonlinear convergence failures
  * during one step try.
  */
 
@@ -450,7 +502,7 @@ int CVodeSetMaxNonlinIters(void *cvode_mem, int maxcor)
   return(CV_SUCCESS);
 }
 
-/* 
+/*
  * CVodeSetNonlinConvCoef
  *
  * Specifies the coeficient in the nonlinear solver convergence
@@ -473,7 +525,39 @@ int CVodeSetNonlinConvCoef(void *cvode_mem, realtype nlscoef)
   return(CV_SUCCESS);
 }
 
-/* 
+/*
+ * CVodeSetLSetupFrequency
+ *
+ * Specifies the frequency for calling the linear solver setup function to
+ * recompute the Jacobian matrix and/or preconditioner
+ */
+
+int CVodeSetLSetupFrequency(void *cvode_mem, long int msbp)
+{
+  CVodeMem cv_mem;
+
+  if (cvode_mem == NULL) {
+    cvProcessError(NULL, CV_MEM_NULL, "CVODES", "CVodeSetLSetupFrequency",
+                   MSGCV_NO_MEM);
+    return(CV_MEM_NULL);
+  }
+
+  cv_mem = (CVodeMem) cvode_mem;
+
+  /* check for a valid input */
+  if (msbp < 0) {
+    cvProcessError(cv_mem, CV_ILL_INPUT, "CVODES", "CVodeSetLSetupFrequency",
+                   "A negative setup frequency was provided");
+    return(CV_ILL_INPUT);
+  }
+
+  /* use default or user provided value */
+  cv_mem->cv_msbp = (msbp == 0) ? MSBP : msbp;
+
+  return(CV_SUCCESS);
+}
+
+/*
  * CVodeSetRootDirection
  *
  * Specifies the direction of zero-crossings to be monitored.
@@ -495,7 +579,7 @@ int CVodeSetRootDirection(void *cvode_mem, int *rootdir)
   nrt = cv_mem->cv_nrtfn;
   if (nrt==0) {
     cvProcessError(cv_mem, CV_ILL_INPUT, "CVODES", "CVodeSetRootDirection", MSGCV_NO_ROOT);
-    return(CV_ILL_INPUT);    
+    return(CV_ILL_INPUT);
   }
 
   for(i=0; i<nrt; i++) cv_mem->cv_rootdir[i] = rootdir[i];
@@ -523,7 +607,7 @@ int CVodeSetNoInactiveRootWarn(void *cvode_mem)
   cv_mem = (CVodeMem) cvode_mem;
 
   cv_mem->cv_mxgnull = 0;
-  
+
   return(CV_SUCCESS);
 }
 
@@ -590,7 +674,7 @@ int CVodeSetConstraints(void *cvode_mem, N_Vector constraints)
   return(CV_SUCCESS);
 }
 
-/* 
+/*
  * =================================================================
  * Quadrature optional input functions
  * =================================================================
@@ -601,7 +685,7 @@ int CVodeSetQuadErrCon(void *cvode_mem, booleantype errconQ)
   CVodeMem cv_mem;
 
   if (cvode_mem==NULL) {
-    cvProcessError(NULL, CV_MEM_NULL, "CVODES", "CVodeSetQuadErrCon", MSGCV_NO_MEM);    
+    cvProcessError(NULL, CV_MEM_NULL, "CVODES", "CVodeSetQuadErrCon", MSGCV_NO_MEM);
     return(CV_MEM_NULL);
   }
   cv_mem = (CVodeMem) cvode_mem;
@@ -611,7 +695,7 @@ int CVodeSetQuadErrCon(void *cvode_mem, booleantype errconQ)
   return(CV_SUCCESS);
 }
 
-/* 
+/*
  * =================================================================
  * FSA optional input functions
  * =================================================================
@@ -622,19 +706,19 @@ int CVodeSetSensDQMethod(void *cvode_mem, int DQtype, realtype DQrhomax)
   CVodeMem cv_mem;
 
   if (cvode_mem==NULL) {
-    cvProcessError(NULL, CV_MEM_NULL, "CVODES", "CVodeSetSensDQMethod", MSGCV_NO_MEM);    
+    cvProcessError(NULL, CV_MEM_NULL, "CVODES", "CVodeSetSensDQMethod", MSGCV_NO_MEM);
     return(CV_MEM_NULL);
   }
 
   cv_mem = (CVodeMem) cvode_mem;
 
   if ( (DQtype != CV_CENTERED) && (DQtype != CV_FORWARD) ) {
-    cvProcessError(cv_mem, CV_ILL_INPUT, "CVODES", "CVodeSetSensDQMethod", MSGCV_BAD_DQTYPE);    
+    cvProcessError(cv_mem, CV_ILL_INPUT, "CVODES", "CVodeSetSensDQMethod", MSGCV_BAD_DQTYPE);
     return(CV_ILL_INPUT);
   }
 
   if (DQrhomax < ZERO ) {
-    cvProcessError(cv_mem, CV_ILL_INPUT, "CVODES", "CVodeSetSensDQMethod", MSGCV_BAD_DQRHO);    
+    cvProcessError(cv_mem, CV_ILL_INPUT, "CVODES", "CVodeSetSensDQMethod", MSGCV_BAD_DQRHO);
     return(CV_ILL_INPUT);
   }
 
@@ -651,7 +735,7 @@ int CVodeSetSensErrCon(void *cvode_mem, booleantype errconS)
   CVodeMem cv_mem;
 
   if (cvode_mem==NULL) {
-    cvProcessError(NULL, CV_MEM_NULL, "CVODES", "CVodeSetSensErrCon", MSGCV_NO_MEM);    
+    cvProcessError(NULL, CV_MEM_NULL, "CVODES", "CVodeSetSensErrCon", MSGCV_NO_MEM);
     return(CV_MEM_NULL);
   }
   cv_mem = (CVodeMem) cvode_mem;
@@ -713,7 +797,7 @@ int CVodeSetSensParams(void *cvode_mem, realtype *p, realtype *pbar, int *plist)
   int is, Ns;
 
   if (cvode_mem==NULL) {
-    cvProcessError(NULL, CV_MEM_NULL, "CVODES", "CVodeSetSensParams", MSGCV_NO_MEM);    
+    cvProcessError(NULL, CV_MEM_NULL, "CVODES", "CVodeSetSensParams", MSGCV_NO_MEM);
     return(CV_MEM_NULL);
   }
 
@@ -772,7 +856,7 @@ int CVodeSetQuadSensErrCon(void *cvode_mem, booleantype errconQS)
   if (cvode_mem==NULL) {
     cvProcessError(NULL, CV_MEM_NULL, "CVODES", "CVodeSetQuadSensErrCon", MSGCV_NO_MEM);
     return(CV_MEM_NULL);
-  }  
+  }
   cv_mem = (CVodeMem) cvode_mem;
 
   /* Was sensitivity initialized? */
@@ -780,12 +864,12 @@ int CVodeSetQuadSensErrCon(void *cvode_mem, booleantype errconQS)
   if (cv_mem->cv_SensMallocDone == SUNFALSE) {
     cvProcessError(cv_mem, CV_NO_SENS, "CVODES", "CVodeSetQuadSensTolerances", MSGCV_NO_SENSI);
     return(CV_NO_SENS);
-  } 
+  }
 
   /* Ckeck if quadrature sensitivity was initialized? */
 
   if (cv_mem->cv_QuadSensMallocDone == SUNFALSE) {
-    cvProcessError(cv_mem, CV_NO_QUADSENS, "CVODES", "CVodeSetQuadSensErrCon", MSGCV_NO_QUADSENSI); 
+    cvProcessError(cv_mem, CV_NO_QUADSENS, "CVODES", "CVodeSetQuadSensErrCon", MSGCV_NO_QUADSENSI);
     return(CV_NO_QUAD);
   }
 
@@ -794,7 +878,7 @@ int CVodeSetQuadSensErrCon(void *cvode_mem, booleantype errconQS)
   return(CV_SUCCESS);
 }
 
-/* 
+/*
  * =================================================================
  * CVODES optional output functions
  * =================================================================
@@ -822,7 +906,7 @@ int CVodeGetNumSteps(void *cvode_mem, long int *nsteps)
   return(CV_SUCCESS);
 }
 
-/* 
+/*
  * CVodeGetNumRhsEvals
  *
  * Returns the current number of calls to f
@@ -844,7 +928,7 @@ int CVodeGetNumRhsEvals(void *cvode_mem, long int *nfevals)
   return(CV_SUCCESS);
 }
 
-/* 
+/*
  * CVodeGetNumLinSolvSetups
  *
  * Returns the current number of calls to the linear solver setup routine
@@ -888,7 +972,7 @@ int CVodeGetNumErrTestFails(void *cvode_mem, long int *netfails)
   return(CV_SUCCESS);
 }
 
-/* 
+/*
  * CVodeGetLastOrder
  *
  * Returns the order on the last succesful step
@@ -910,7 +994,7 @@ int CVodeGetLastOrder(void *cvode_mem, int *qlast)
   return(CV_SUCCESS);
 }
 
-/* 
+/*
  * CVodeGetCurrentOrder
  *
  * Returns the order to be attempted on the next step
@@ -932,7 +1016,29 @@ int CVodeGetCurrentOrder(void *cvode_mem, int *qcur)
   return(CV_SUCCESS);
 }
 
-/* 
+/*
+ * CVodeGetCurrentGamma
+ *
+ * Returns the value of gamma for the current step.
+ */
+
+int CVodeGetCurrentGamma(void *cvode_mem, realtype *gamma)
+{
+  CVodeMem cv_mem;
+
+  if (cvode_mem==NULL) {
+    cvProcessError(NULL, CV_MEM_NULL, "CVODES", "CVodeGetCurrentGamma", MSGCV_NO_MEM);
+    return(CV_MEM_NULL);
+  }
+
+  cv_mem = (CVodeMem) cvode_mem;
+
+  *gamma = cv_mem->cv_gamma;
+
+  return(CV_SUCCESS);
+}
+
+/*
  * CVodeGetNumStabLimOrderReds
  *
  * Returns the number of order reductions triggered by the stability
@@ -958,7 +1064,7 @@ int CVodeGetNumStabLimOrderReds(void *cvode_mem, long int *nslred)
   return(CV_SUCCESS);
 }
 
-/* 
+/*
  * CVodeGetActualInitStep
  *
  * Returns the step size used on the first step
@@ -1002,7 +1108,7 @@ int CVodeGetLastStep(void *cvode_mem, realtype *hlast)
   return(CV_SUCCESS);
 }
 
-/* 
+/*
  * CVodeGetCurrentStep
  *
  * Returns the step size to be attempted on the next step
@@ -1018,13 +1124,80 @@ int CVodeGetCurrentStep(void *cvode_mem, realtype *hcur)
   }
 
   cv_mem = (CVodeMem) cvode_mem;
-  
+
   *hcur = cv_mem->cv_next_h;
 
   return(CV_SUCCESS);
 }
 
-/* 
+/*
+ * CVodeGetCurrentState
+ *
+ * Returns the current state vector
+ */
+
+int CVodeGetCurrentState(void *cvode_mem, N_Vector *y)
+{
+  CVodeMem cv_mem;
+
+  if (cvode_mem==NULL) {
+    cvProcessError(NULL, CV_MEM_NULL, "CVODES", "CVodeGetCurrentState", MSGCV_NO_MEM);
+    return(CV_MEM_NULL);
+  }
+
+  cv_mem = (CVodeMem) cvode_mem;
+
+  *y = cv_mem->cv_y;
+
+  return(CV_SUCCESS);
+}
+
+/*
+ * CVodeGetCurrentStateSens
+ *
+ * Returns the current sensitivity state vector array
+ */
+
+int CVodeGetCurrentStateSens(void *cvode_mem, N_Vector **yS)
+{
+  CVodeMem cv_mem;
+
+  if (cvode_mem==NULL) {
+    cvProcessError(NULL, CV_MEM_NULL, "CVODES", "CVodeGetCurrentStateSens", MSGCV_NO_MEM);
+    return(CV_MEM_NULL);
+  }
+
+  cv_mem = (CVodeMem) cvode_mem;
+
+  *yS = cv_mem->cv_yS;
+
+  return(CV_SUCCESS);
+}
+
+/*
+ * CVodeGetCurrentSensSolveIndex
+ *
+ * Returns the current index of the sensitivity solve when using
+ * the staggered1 nonlinear solver.
+ */
+
+int CVodeGetCurrentSensSolveIndex(void *cvode_mem, int *index)
+{
+  CVodeMem cv_mem;
+
+  if (cvode_mem==NULL) {
+    cvProcessError(NULL, CV_MEM_NULL, "CVODES", "CVodeGetCurrentSensSolveIndex", MSGCV_NO_MEM);
+    return(CV_MEM_NULL);
+  }
+
+  cv_mem = (CVodeMem) cvode_mem;
+
+  *index = cv_mem->sens_solve_idx;
+
+  return(CV_SUCCESS);
+}
+
+/*
  * CVodeGetCurrentTime
  *
  * Returns the current value of the independent variable
@@ -1046,7 +1219,7 @@ int CVodeGetCurrentTime(void *cvode_mem, realtype *tcur)
   return(CV_SUCCESS);
 }
 
-/* 
+/*
  * CVodeGetTolScaleFactor
  *
  * Returns a suggested factor for scaling tolerances
@@ -1068,7 +1241,7 @@ int CVodeGetTolScaleFactor(void *cvode_mem, realtype *tolsfact)
   return(CV_SUCCESS);
 }
 
-/* 
+/*
  * CVodeGetErrWeights
  *
  * This routine returns the current weight vector.
@@ -1112,7 +1285,7 @@ int CVodeGetEstLocalErrors(void *cvode_mem, N_Vector ele)
   return(CV_SUCCESS);
 }
 
-/* 
+/*
  * CVodeGetWorkSpace
  *
  * Returns integrator work space requirements
@@ -1135,15 +1308,15 @@ int CVodeGetWorkSpace(void *cvode_mem, long int *lenrw, long int *leniw)
   return(CV_SUCCESS);
 }
 
-/* 
+/*
  * CVodeGetIntegratorStats
  *
  * Returns integrator statistics
  */
 
-int CVodeGetIntegratorStats(void *cvode_mem, long int *nsteps, long int *nfevals, 
-                            long int *nlinsetups, long int *netfails, int *qlast, 
-                            int *qcur, realtype *hinused, realtype *hlast, 
+int CVodeGetIntegratorStats(void *cvode_mem, long int *nsteps, long int *nfevals,
+                            long int *nlinsetups, long int *netfails, int *qlast,
+                            int *qcur, realtype *hinused, realtype *hlast,
                             realtype *hcur, realtype *tcur)
 {
   CVodeMem cv_mem;
@@ -1169,7 +1342,7 @@ int CVodeGetIntegratorStats(void *cvode_mem, long int *nsteps, long int *nfevals
   return(CV_SUCCESS);
 }
 
-/* 
+/*
  * CVodeGetNumGEvals
  *
  * Returns the current number of calls to g (for rootfinding)
@@ -1191,7 +1364,7 @@ int CVodeGetNumGEvals(void *cvode_mem, long int *ngevals)
   return(CV_SUCCESS);
 }
 
-/* 
+/*
  * CVodeGetRootInfo
  *
  * Returns pointer to array rootsfound showing roots found
@@ -1226,7 +1399,6 @@ int CVodeGetRootInfo(void *cvode_mem, int *rootsfound)
 int CVodeGetNumNonlinSolvIters(void *cvode_mem, long int *nniters)
 {
   CVodeMem cv_mem;
-  booleantype sensi_sim;
 
   if (cvode_mem==NULL) {
     cvProcessError(NULL, CV_MEM_NULL, "CVODES",
@@ -1236,37 +1408,12 @@ int CVodeGetNumNonlinSolvIters(void *cvode_mem, long int *nniters)
 
   cv_mem = (CVodeMem) cvode_mem;
 
-  /* are we computing sensitivities with the simultaneous approach? */
-  sensi_sim = (cv_mem->cv_sensi && (cv_mem->cv_ism==CV_SIMULTANEOUS));
-
-  /* get number of iterations from the NLS */
-  if (sensi_sim) {
-
-    /* check that the NLS is non-NULL */
-    if (cv_mem->NLSsim == NULL) {
-      cvProcessError(NULL, CV_MEM_FAIL, "CVODES",
-                     "CVodeGetNumNonlinSolvIters", MSGCV_MEM_FAIL);
-      return(CV_MEM_FAIL);
-    }
-
-    return(SUNNonlinSolGetNumIters(cv_mem->NLSsim, nniters));
-
-  } else {
-
-    /* check that the NLS is non-NULL */
-    if (cv_mem->NLS == NULL) {
-      cvProcessError(NULL, CV_MEM_FAIL, "CVODES",
-                     "CVodeGetNumNonlinSolvIters", MSGCV_MEM_FAIL);
-      return(CV_MEM_FAIL);
-    }
-
-    return(SUNNonlinSolGetNumIters(cv_mem->NLS, nniters));
-  }
+  *nniters = cv_mem->cv_nni;
 
   return(CV_SUCCESS);
 }
 
-/* 
+/*
  * CVodeGetNumNonlinSolvConvFails
  *
  * Returns the current number of convergence failures in the
@@ -1299,7 +1446,6 @@ int CVodeGetNonlinSolvStats(void *cvode_mem, long int *nniters,
                             long int *nncfails)
 {
   CVodeMem cv_mem;
-  booleantype sensi_sim;
 
   if (cvode_mem==NULL) {
     cvProcessError(NULL, CV_MEM_NULL, "CVODES",
@@ -1309,40 +1455,14 @@ int CVodeGetNonlinSolvStats(void *cvode_mem, long int *nniters,
 
   cv_mem = (CVodeMem) cvode_mem;
 
+  *nniters  = cv_mem->cv_nni;
   *nncfails = cv_mem->cv_ncfn;
 
-  /* are we computing sensitivities with the simultaneous approach? */
-  sensi_sim = (cv_mem->cv_sensi && (cv_mem->cv_ism==CV_SIMULTANEOUS));
-
-  /* get number of iterations from the NLS */
-  if (sensi_sim) {
-
-    /* check that the NLS is non-NULL */
-    if (cv_mem->NLSsim == NULL) {
-      cvProcessError(NULL, CV_MEM_FAIL, "CVODES",
-                     "CVodeGetNumNonlinSolvIters", MSGCV_MEM_FAIL);
-      return(CV_MEM_FAIL);
-    }
-
-    return(SUNNonlinSolGetNumIters(cv_mem->NLSsim, nniters));
-
-  } else {
-
-    /* check that the NLS is non-NULL */
-    if (cv_mem->NLS == NULL) {
-      cvProcessError(NULL, CV_MEM_FAIL, "CVODES",
-                     "CVodeGetNumNonlinSolvIters", MSGCV_MEM_FAIL);
-      return(CV_MEM_FAIL);
-    }
-
-    return(SUNNonlinSolGetNumIters(cv_mem->NLS, nniters));
-  }
-
   return(CV_SUCCESS);
 }
 
 
-/* 
+/*
  * =================================================================
  * Quadrature optional output functions
  * =================================================================
@@ -1355,14 +1475,14 @@ int CVodeGetQuadNumRhsEvals(void *cvode_mem, long int *nfQevals)
   CVodeMem cv_mem;
 
   if (cvode_mem==NULL) {
-    cvProcessError(NULL, CV_MEM_NULL, "CVODES", "CVodeGetQuadNumRhsEvals", MSGCV_NO_MEM);    
+    cvProcessError(NULL, CV_MEM_NULL, "CVODES", "CVodeGetQuadNumRhsEvals", MSGCV_NO_MEM);
     return(CV_MEM_NULL);
   }
 
   cv_mem = (CVodeMem) cvode_mem;
 
   if (cv_mem->cv_quadr==SUNFALSE) {
-    cvProcessError(cv_mem, CV_NO_QUAD, "CVODES", "CVodeGetQuadNumRhsEvals", MSGCV_NO_QUAD); 
+    cvProcessError(cv_mem, CV_NO_QUAD, "CVODES", "CVodeGetQuadNumRhsEvals", MSGCV_NO_QUAD);
     return(CV_NO_QUAD);
   }
 
@@ -1378,14 +1498,14 @@ int CVodeGetQuadNumErrTestFails(void *cvode_mem, long int *nQetfails)
   CVodeMem cv_mem;
 
   if (cvode_mem==NULL) {
-    cvProcessError(NULL, CV_MEM_NULL, "CVODES", "CVodeGetQuadNumErrTestFails", MSGCV_NO_MEM);    
+    cvProcessError(NULL, CV_MEM_NULL, "CVODES", "CVodeGetQuadNumErrTestFails", MSGCV_NO_MEM);
     return(CV_MEM_NULL);
   }
 
   cv_mem = (CVodeMem) cvode_mem;
 
   if (cv_mem->cv_quadr==SUNFALSE) {
-    cvProcessError(cv_mem, CV_NO_QUAD, "CVODES", "CVodeGetQuadNumErrTestFails", MSGCV_NO_QUAD); 
+    cvProcessError(cv_mem, CV_NO_QUAD, "CVODES", "CVodeGetQuadNumErrTestFails", MSGCV_NO_QUAD);
     return(CV_NO_QUAD);
   }
 
@@ -1401,14 +1521,14 @@ int CVodeGetQuadErrWeights(void *cvode_mem, N_Vector eQweight)
   CVodeMem cv_mem;
 
   if (cvode_mem==NULL) {
-    cvProcessError(NULL, CV_MEM_NULL, "CVODES", "CVodeGetQuadErrWeights", MSGCV_NO_MEM); 
+    cvProcessError(NULL, CV_MEM_NULL, "CVODES", "CVodeGetQuadErrWeights", MSGCV_NO_MEM);
     return(CV_MEM_NULL);
   }
 
   cv_mem = (CVodeMem) cvode_mem;
 
   if (cv_mem->cv_quadr==SUNFALSE) {
-    cvProcessError(cv_mem, CV_NO_QUAD, "CVODES", "CVodeGetQuadErrWeights", MSGCV_NO_QUAD); 
+    cvProcessError(cv_mem, CV_NO_QUAD, "CVODES", "CVodeGetQuadErrWeights", MSGCV_NO_QUAD);
     return(CV_NO_QUAD);
   }
 
@@ -1424,14 +1544,14 @@ int CVodeGetQuadStats(void *cvode_mem, long int *nfQevals, long int *nQetfails)
   CVodeMem cv_mem;
 
   if (cvode_mem==NULL) {
-    cvProcessError(NULL, CV_MEM_NULL, "CVODES", "CVodeGetQuadStats", MSGCV_NO_MEM);    
+    cvProcessError(NULL, CV_MEM_NULL, "CVODES", "CVodeGetQuadStats", MSGCV_NO_MEM);
     return(CV_MEM_NULL);
   }
 
   cv_mem = (CVodeMem) cvode_mem;
 
   if (cv_mem->cv_quadr==SUNFALSE) {
-    cvProcessError(cv_mem, CV_NO_QUAD, "CVODES", "CVodeGetQuadStats", MSGCV_NO_QUAD); 
+    cvProcessError(cv_mem, CV_NO_QUAD, "CVODES", "CVodeGetQuadStats", MSGCV_NO_QUAD);
     return(CV_NO_QUAD);
   }
 
@@ -1441,7 +1561,7 @@ int CVodeGetQuadStats(void *cvode_mem, long int *nfQevals, long int *nQetfails)
   return(CV_SUCCESS);
 }
 
-/* 
+/*
  * =================================================================
  * Quadrature FSA optional output functions
  * =================================================================
@@ -1454,14 +1574,14 @@ int CVodeGetQuadSensNumRhsEvals(void *cvode_mem, long int *nfQSevals)
   CVodeMem cv_mem;
 
   if (cvode_mem==NULL) {
-    cvProcessError(NULL, CV_MEM_NULL, "CVODES", "CVodeGetQuadSensNumRhsEvals", MSGCV_NO_MEM);    
+    cvProcessError(NULL, CV_MEM_NULL, "CVODES", "CVodeGetQuadSensNumRhsEvals", MSGCV_NO_MEM);
     return(CV_MEM_NULL);
   }
 
   cv_mem = (CVodeMem) cvode_mem;
 
   if (cv_mem->cv_quadr_sensi == SUNFALSE) {
-    cvProcessError(cv_mem, CV_NO_QUADSENS, "CVODES", "CVodeGetQuadSensNumRhsEvals", MSGCV_NO_QUADSENSI); 
+    cvProcessError(cv_mem, CV_NO_QUADSENS, "CVODES", "CVodeGetQuadSensNumRhsEvals", MSGCV_NO_QUADSENSI);
     return(CV_NO_QUADSENS);
   }
 
@@ -1477,14 +1597,14 @@ int CVodeGetQuadSensNumErrTestFails(void *cvode_mem, long int *nQSetfails)
   CVodeMem cv_mem;
 
   if (cvode_mem==NULL) {
-    cvProcessError(NULL, CV_MEM_NULL, "CVODES", "CVodeGetQuadSensNumErrTestFails", MSGCV_NO_MEM);    
+    cvProcessError(NULL, CV_MEM_NULL, "CVODES", "CVodeGetQuadSensNumErrTestFails", MSGCV_NO_MEM);
     return(CV_MEM_NULL);
   }
 
   cv_mem = (CVodeMem) cvode_mem;
 
   if (cv_mem->cv_quadr_sensi == SUNFALSE) {
-    cvProcessError(cv_mem, CV_NO_QUADSENS, "CVODES", "CVodeGetQuadSensNumErrTestFails", MSGCV_NO_QUADSENSI); 
+    cvProcessError(cv_mem, CV_NO_QUADSENS, "CVODES", "CVodeGetQuadSensNumErrTestFails", MSGCV_NO_QUADSENSI);
     return(CV_NO_QUADSENS);
   }
 
@@ -1501,14 +1621,14 @@ int CVodeGetQuadSensErrWeights(void *cvode_mem, N_Vector *eQSweight)
   int is, Ns;
 
   if (cvode_mem==NULL) {
-    cvProcessError(NULL, CV_MEM_NULL, "CVODES", "CVodeGetQuadSensErrWeights", MSGCV_NO_MEM); 
+    cvProcessError(NULL, CV_MEM_NULL, "CVODES", "CVodeGetQuadSensErrWeights", MSGCV_NO_MEM);
     return(CV_MEM_NULL);
   }
 
   cv_mem = (CVodeMem) cvode_mem;
 
   if (cv_mem->cv_quadr_sensi == SUNFALSE) {
-    cvProcessError(cv_mem, CV_NO_QUADSENS, "CVODES", "CVodeGetQuadSensErrWeights", MSGCV_NO_QUADSENSI); 
+    cvProcessError(cv_mem, CV_NO_QUADSENS, "CVODES", "CVodeGetQuadSensErrWeights", MSGCV_NO_QUADSENSI);
     return(CV_NO_QUADSENS);
   }
   Ns = cv_mem->cv_Ns;
@@ -1527,14 +1647,14 @@ int CVodeGetQuadSensStats(void *cvode_mem, long int *nfQSevals, long int *nQSetf
   CVodeMem cv_mem;
 
   if (cvode_mem==NULL) {
-    cvProcessError(NULL, CV_MEM_NULL, "CVODES", "CVodeGetQuadSensStats", MSGCV_NO_MEM);    
+    cvProcessError(NULL, CV_MEM_NULL, "CVODES", "CVodeGetQuadSensStats", MSGCV_NO_MEM);
     return(CV_MEM_NULL);
   }
 
   cv_mem = (CVodeMem) cvode_mem;
 
   if (cv_mem->cv_quadr_sensi == SUNFALSE) {
-    cvProcessError(cv_mem, CV_NO_QUADSENS, "CVODES", "CVodeGetQuadSensStats", MSGCV_NO_QUADSENSI); 
+    cvProcessError(cv_mem, CV_NO_QUADSENS, "CVODES", "CVodeGetQuadSensStats", MSGCV_NO_QUADSENSI);
     return(CV_NO_QUADSENS);
   }
 
@@ -1545,7 +1665,7 @@ int CVodeGetQuadSensStats(void *cvode_mem, long int *nfQSevals, long int *nQSetf
 }
 
 
-/* 
+/*
  * =================================================================
  * FSA optional output functions
  * =================================================================
@@ -1558,7 +1678,7 @@ int CVodeGetSensNumRhsEvals(void *cvode_mem, long int *nfSevals)
   CVodeMem cv_mem;
 
   if (cvode_mem==NULL) {
-    cvProcessError(NULL, CV_MEM_NULL, "CVODES", "CVodeGetSensNumRhsEvals", MSGCV_NO_MEM);    
+    cvProcessError(NULL, CV_MEM_NULL, "CVODES", "CVodeGetSensNumRhsEvals", MSGCV_NO_MEM);
     return(CV_MEM_NULL);
   }
 
@@ -1581,7 +1701,7 @@ int CVodeGetNumRhsEvalsSens(void *cvode_mem, long int *nfevalsS)
   CVodeMem cv_mem;
 
   if (cvode_mem==NULL) {
-    cvProcessError(NULL, CV_MEM_NULL, "CVODES", "CVodeGetNumRhsEvalsSens", MSGCV_NO_MEM);    
+    cvProcessError(NULL, CV_MEM_NULL, "CVODES", "CVodeGetNumRhsEvalsSens", MSGCV_NO_MEM);
     return(CV_MEM_NULL);
   }
 
@@ -1604,7 +1724,7 @@ int CVodeGetSensNumErrTestFails(void *cvode_mem, long int *nSetfails)
   CVodeMem cv_mem;
 
   if (cvode_mem==NULL) {
-    cvProcessError(NULL, CV_MEM_NULL, "CVODES", "CVodeGetSensNumErrTestFails", MSGCV_NO_MEM);    
+    cvProcessError(NULL, CV_MEM_NULL, "CVODES", "CVodeGetSensNumErrTestFails", MSGCV_NO_MEM);
     return(CV_MEM_NULL);
   }
 
@@ -1627,7 +1747,7 @@ int CVodeGetSensNumLinSolvSetups(void *cvode_mem, long int *nlinsetupsS)
   CVodeMem cv_mem;
 
   if (cvode_mem==NULL) {
-    cvProcessError(NULL, CV_MEM_NULL, "CVODES", "CVodeGetSensNumLinSolvSetups", MSGCV_NO_MEM);    
+    cvProcessError(NULL, CV_MEM_NULL, "CVODES", "CVodeGetSensNumLinSolvSetups", MSGCV_NO_MEM);
     return(CV_MEM_NULL);
   }
 
@@ -1651,7 +1771,7 @@ int CVodeGetSensErrWeights(void *cvode_mem, N_Vector *eSweight)
   int is, Ns;
 
   if (cvode_mem==NULL) {
-    cvProcessError(NULL, CV_MEM_NULL, "CVODES", "CVodeGetSensErrWeights", MSGCV_NO_MEM);    
+    cvProcessError(NULL, CV_MEM_NULL, "CVODES", "CVodeGetSensErrWeights", MSGCV_NO_MEM);
     return(CV_MEM_NULL);
   }
 
@@ -1672,7 +1792,7 @@ int CVodeGetSensErrWeights(void *cvode_mem, N_Vector *eSweight)
 
 /*-----------------------------------------------------------------*/
 
-int CVodeGetSensStats(void *cvode_mem, long int *nfSevals, long int *nfevalsS, 
+int CVodeGetSensStats(void *cvode_mem, long int *nfSevals, long int *nfevalsS,
                       long int *nSetfails, long int *nlinsetupsS)
 {
   CVodeMem cv_mem;
@@ -1702,7 +1822,6 @@ int CVodeGetSensStats(void *cvode_mem, long int *nfSevals, long int *nfevalsS,
 int CVodeGetSensNumNonlinSolvIters(void *cvode_mem, long int *nSniters)
 {
   CVodeMem cv_mem;
-  booleantype sensi_stg;
 
   if (cvode_mem==NULL) {
     cvProcessError(NULL, CV_MEM_NULL, "CVODES",
@@ -1718,32 +1837,9 @@ int CVodeGetSensNumNonlinSolvIters(void *cvode_mem, long int *nSniters)
     return(CV_NO_SENS);
   }
 
-  /* Are we computing sensitivities with a staggered approach? */
-  sensi_stg = (cv_mem->cv_sensi && (cv_mem->cv_ism==CV_STAGGERED));
-
-  if (sensi_stg) {
-
-    /* check that the NLS is non-NULL */
-    if (cv_mem->NLSstg == NULL) {
-      cvProcessError(NULL, CV_MEM_FAIL, "CVODES",
-                      "CVodeGetSensNumNonlinSolvIters", MSGCV_MEM_FAIL);
-      return(CV_MEM_FAIL);
-    }
-
-    return(SUNNonlinSolGetNumIters(cv_mem->NLSstg, nSniters));
-
-  } else {
-
-    /* check that the NLS is non-NULL */
-    if (cv_mem->NLSstg1 == NULL) {
-      cvProcessError(NULL, CV_MEM_FAIL, "CVODES",
-                      "CVodeGetSensNumNonlinSolvIters", MSGCV_MEM_FAIL);
-      return(CV_MEM_FAIL);
-    }
-
-    return(SUNNonlinSolGetNumIters(cv_mem->NLSstg1, nSniters));
-  }
+  *nSniters = cv_mem->cv_nniS;
 
+  return(CV_SUCCESS);
 }
 
 /*-----------------------------------------------------------------*/
@@ -1771,6 +1867,33 @@ int CVodeGetSensNumNonlinSolvConvFails(void *cvode_mem, long int *nSncfails)
 
 /*-----------------------------------------------------------------*/
 
+int CVodeGetSensNonlinSolvStats(void *cvode_mem, long int *nSniters,
+                                long int *nSncfails)
+{
+  CVodeMem cv_mem;
+
+  if (cvode_mem==NULL) {
+    cvProcessError(NULL, CV_MEM_NULL, "CVODES",
+                   "CVodeGetSensNonlinSolvStats", MSGCV_NO_MEM);
+    return(CV_MEM_NULL);
+  }
+
+  cv_mem = (CVodeMem) cvode_mem;
+
+  if (cv_mem->cv_sensi==SUNFALSE) {
+    cvProcessError(cv_mem, CV_NO_SENS, "CVODES",
+                   "CVodeGetSensNonlinSolvStats", MSGCV_NO_SENSI);
+    return(CV_NO_SENS);
+  }
+
+  *nSniters  = cv_mem->cv_nniS;
+  *nSncfails = cv_mem->cv_ncfnS;
+
+  return(CV_SUCCESS);
+}
+
+/*-----------------------------------------------------------------*/
+
 int CVodeGetStgrSensNumNonlinSolvIters(void *cvode_mem, long int *nSTGR1niters)
 {
   CVodeMem cv_mem;
@@ -1790,7 +1913,7 @@ int CVodeGetStgrSensNumNonlinSolvIters(void *cvode_mem, long int *nSTGR1niters)
     return(CV_NO_SENS);
   }
 
-  if(cv_mem->cv_ism==CV_STAGGERED1) 
+  if(cv_mem->cv_ism==CV_STAGGERED1)
     for(is=0; is<Ns; is++) nSTGR1niters[is] = cv_mem->cv_nniS1[is];
 
   return(CV_SUCCESS);
@@ -1817,7 +1940,7 @@ int CVodeGetStgrSensNumNonlinSolvConvFails(void *cvode_mem, long int *nSTGR1ncfa
     return(CV_NO_SENS);
   }
 
-  if(cv_mem->cv_ism==CV_STAGGERED1) 
+  if(cv_mem->cv_ism==CV_STAGGERED1)
     for(is=0; is<Ns; is++) nSTGR1ncfails[is] = cv_mem->cv_ncfnS1[is];
 
   return(CV_SUCCESS);
@@ -1825,52 +1948,32 @@ int CVodeGetStgrSensNumNonlinSolvConvFails(void *cvode_mem, long int *nSTGR1ncfa
 
 /*-----------------------------------------------------------------*/
 
-int CVodeGetSensNonlinSolvStats(void *cvode_mem, long int *nSniters, 
-                                long int *nSncfails)
+int CVodeGetStgrSensNonlinSolvStats(void *cvode_mem,
+                                    long int *nSTGR1niters,
+                                    long int *nSTGR1ncfails)
 {
   CVodeMem cv_mem;
-  booleantype sensi_stg;
+  int is, Ns;
 
-  if (cvode_mem==NULL) {
+  if (cvode_mem == NULL) {
     cvProcessError(NULL, CV_MEM_NULL, "CVODES",
-                   "CVodeGetSensNonlinSolvstats", MSGCV_NO_MEM);
+                   "CVodeGetStgrSensNonlinSolvStats", MSGCV_NO_MEM);
     return(CV_MEM_NULL);
   }
 
   cv_mem = (CVodeMem) cvode_mem;
 
-  if (cv_mem->cv_sensi==SUNFALSE) {
+  Ns = cv_mem->cv_Ns;
+
+  if (cv_mem->cv_sensi == SUNFALSE) {
     cvProcessError(cv_mem, CV_NO_SENS, "CVODES",
-                   "CVodeGetSensNonlinSolvStats", MSGCV_NO_SENSI);
+                   "CVodeGetStgrSensNonlinSolvStats", MSGCV_NO_SENSI);
     return(CV_NO_SENS);
   }
 
-  *nSncfails = cv_mem->cv_ncfnS;
-
-  /* Are we computing sensitivities with a staggered approach? */
-  sensi_stg = (cv_mem->cv_sensi && (cv_mem->cv_ism==CV_STAGGERED));
-
-  if (sensi_stg) {
-
-    /* check that the NLS is non-NULL */
-    if (cv_mem->NLSstg == NULL) {
-      cvProcessError(NULL, CV_MEM_FAIL, "CVODES",
-                      "CVodeGetSensNumNonlinSolvStats", MSGCV_MEM_FAIL);
-      return(CV_MEM_FAIL);
-    }
-
-    return(SUNNonlinSolGetNumIters(cv_mem->NLSstg, nSniters));
-
-  } else {
-
-    /* check that the NLS is non-NULL */
-    if (cv_mem->NLSstg1 == NULL) {
-      cvProcessError(NULL, CV_MEM_FAIL, "CVODES",
-                      "CVodeGetSensNumNonlinSolvStats", MSGCV_MEM_FAIL);
-      return(CV_MEM_FAIL);
-    }
-
-    return(SUNNonlinSolGetNumIters(cv_mem->NLSstg1, nSniters));
+  if(cv_mem->cv_ism == CV_STAGGERED1) {
+    for(is=0; is<Ns; is++) nSTGR1niters[is]  = cv_mem->cv_nniS1[is];
+    for(is=0; is<Ns; is++) nSTGR1ncfails[is] = cv_mem->cv_ncfnS1[is];
   }
 
   return(CV_SUCCESS);
@@ -1986,7 +2089,7 @@ char *CVodeGetReturnFlagName(long int flag)
     break;
   case CV_TOO_CLOSE:
     sprintf(name,"CV_TOO_CLOSE");
-    break; 
+    break;
   case CV_NO_ADJ:
     sprintf(name,"CV_NO_ADJ");
     break;
@@ -2007,11 +2110,13 @@ char *CVodeGetReturnFlagName(long int flag)
     break;
   case CV_GETY_BADT:
     sprintf(name,"CV_GETY_BADT");
-    break;  
+    break;
+  case CV_NLS_FAIL:
+    sprintf(name,"CV_NLS_FAIL");
+    break;
   default:
     sprintf(name,"NONE");
   }
 
   return(name);
 }
-
